//+------------------------------------------------------------------+
//|                                                        e-TFL.mq4 |
//|                                             . aka KimIV |
//|                                              http://www.kimiv.ru |
//|                                                                  |
//|  10.07.2006  Trade From Lines (  ). FreeWare.     |
//+------------------------------------------------------------------+
#property copyright "  . aka KimIV"
#property link      "http://www.kimiv.ru"

//-------    --------------------------------
extern string _P_Expert = "----------  ";
extern string esNameUpLine  = "UPLine";//   
extern string esNameDnLine  = "DNLine";//   
extern bool   ebBuyFromUp   = False;   //    
extern bool   ebBuyFromDn   = False;   //    
extern bool   ebShowComment = True;    //  

extern string _P_Trade = "----------  ";
extern double Lots         = 0.1;      //   
extern int    StopLoss     = 15;       //   
extern int    TakeProfit   = 15;       //   
extern int    Slippage     = 3;        //  

//-------    ----------------------------
string Name_Expert   = "e-TFL";
bool   UseSound      = True;           //   
string NameFileSound = "expert.wav";   //   
color  clOpenBuy     = LightBlue;      //    
color  clOpenSell    = LightCoral;     //    
color  clCloseBuy    = Blue;           //    
color  clCloseSell   = Red;            //    
double gdUpPrice, gdDnPrice;           //   
double gdPrevUpPrice, gdPrevDnPrice;   //   
double gdPrevAsk, gdPrevBid;           //  


//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
void deinit() { if (!IsTesting()) Comment(""); }

//+------------------------------------------------------------------+
//| expert program start function                                    |
//+------------------------------------------------------------------+
void start() {
  gdUpPrice=0;
  gdDnPrice=0;
  if (idPriceLevel() && gdPrevAsk!=0 && gdPrevBid!=0) OpenPositions();

  if (ebShowComment) {
    string st="";
    if (gdUpPrice==0) st="    \n";
    else {
      if (ebBuyFromUp) st="  ";
      else st="  ";
      st=st+DoubleToStr(gdUpPrice, Digits)+"\n";
    }
    if (gdDnPrice==0) st=st+"    ";
    else {
      if (ebBuyFromDn) st=st+"  ";
      else st=st+"  ";
      st=st+DoubleToStr(gdDnPrice, Digits);
    }
    Comment(st);
  }

  if (gdUpPrice>0) gdPrevUpPrice=gdUpPrice;
  if (gdDnPrice>0) gdPrevDnPrice=gdDnPrice;
  gdPrevAsk=Ask;
  gdPrevBid=Bid;
}

//+------------------------------------------------------------------+
//|                       |
//+------------------------------------------------------------------+
bool idPriceLevel() {
  //   
  if (ObjectFind(esNameUpLine)<0) {
    gdUpPrice=0;
    gdPrevUpPrice=0;
    Message("   "+esNameUpLine);
  } else {
    switch (ObjectType(esNameUpLine)) {
      case OBJ_HLINE: gdUpPrice=ObjectGet(esNameUpLine, OBJPROP_PRICE1); break;
      case OBJ_TREND: gdUpPrice=ObjectGetValueByShift(esNameUpLine, 0); break;
      default: Message("   "+esNameUpLine); break;
    }
  }

  //   
  if (ObjectFind(esNameDnLine)<0) {
    gdDnPrice=0;
    gdPrevDnPrice=0;
    Message("   "+esNameDnLine);
  } else {
    switch (ObjectType(esNameDnLine)) {
      case OBJ_HLINE: gdDnPrice=ObjectGet(esNameDnLine, OBJPROP_PRICE1); break;
      case OBJ_TREND: gdDnPrice=ObjectGetValueByShift(esNameDnLine, 0); break;
      default: Message("   "+esNameDnLine); break;
    }
  }

  if (gdUpPrice<gdPrevBid && gdUpPrice>0) {
    gdUpPrice=0;
    Message("  "+esNameUpLine+"   ");
  }
  if (gdDnPrice>gdPrevAsk && gdPrevAsk>0) {
    gdDnPrice=0;
    Message("  "+esNameDnLine+"   ");
  }

  if (gdUpPrice!=gdPrevUpPrice || gdDnPrice!=gdPrevDnPrice
   || gdPrevUpPrice==0 || gdPrevDnPrice==0
   || gdUpPrice==0 || gdUpPrice==0) return(False);
  else return(True);
}

//+------------------------------------------------------------------+
//|                                                   |
//+------------------------------------------------------------------+
void OpenPositions() {
  bool   lbBuy=False, lbSell=False;
  double ldStop=0, ldTake=0;

  if ((Ask>gdPrevAsk && Ask>=gdUpPrice && ebBuyFromUp)
   || (Ask<gdPrevAsk && Ask<=gdDnPrice && ebBuyFromDn)) lbBuy=True;
  if ((Bid>gdPrevBid && Bid>=gdUpPrice && !ebBuyFromUp)
   || (Bid<gdPrevBid && Bid<=gdDnPrice && !ebBuyFromDn)) lbSell=True;

  if (lbBuy) {
    if (ExistPositionByOP(OP_SELL)) CloseAllPositions();
    if (!ExistPositionByOP(OP_BUY)) {
      if (StopLoss!=0) ldStop=Ask-StopLoss*Point;
      if (TakeProfit!=0) ldTake=Ask+TakeProfit*Point;
      OpenPosition(OP_BUY, ldStop, ldTake);
    }
  }
  if (lbSell) {
    if (ExistPositionByOP(OP_BUY)) CloseAllPositions();
    if (!ExistPositionByOP(OP_SELL)) {
      if (StopLoss!=0) ldStop=Bid+StopLoss*Point;
      if (TakeProfit!=0) ldTake=Bid-TakeProfit*Point;
      OpenPosition(OP_SELL, ldStop, ldTake);
    }
  }
}

//+------------------------------------------------------------------+
//|                      |
//+------------------------------------------------------------------+
bool ExistPositionByOP(int op) {
  for (int i=0; i<OrdersTotal(); i++) {
    if (OrderSelect(i, SELECT_BY_POS, MODE_TRADES)) {
      if (OrderSymbol()==Symbol() && OrderType()==op) return(True);
    }
  }
  return(False);
}

//+------------------------------------------------------------------+
//|                                      |
//| :                                                       |
//|   op     -                                               |
//|   ldStop -                                            |
//|   ldTake -                                            |
//+------------------------------------------------------------------+
void OpenPosition(int op, double ldStop, double ldTake) {
  color  clOpen;
  double pp;
  string lsComm=GetCommentForOrder();

  if (op==OP_BUY) {
    clOpen=clOpenBuy;
    pp=Ask;
  } else {
    clOpen=clOpenSell;
    pp=Bid;
  }
  OrderSend(Symbol(),op,Lots,pp,Slippage,ldStop,ldTake,lsComm,0,0,clOpen);
  if (UseSound) PlaySound(NameFileSound);
}

//+------------------------------------------------------------------+
//|          |
//+------------------------------------------------------------------+
string GetCommentForOrder() {
  return(Name_Expert+" "+GetNameTF(Period()));
}

//+------------------------------------------------------------------+
//|                                  |
//+------------------------------------------------------------------+
string GetNameTF(int TimeFrame) {
	switch (TimeFrame) {
		case PERIOD_M1:  return("M1");
		case PERIOD_M5:  return("M5");
		case PERIOD_M15: return("M15");
		case PERIOD_M30: return("M30");
		case PERIOD_H1:  return("H1");
		case PERIOD_H4:  return("H4");
		case PERIOD_D1:  return("Daily");
		case PERIOD_W1:  return("Weekly");
		case PERIOD_MN1: return("Monthly");
		default:		     return("UnknownPeriod");
	}
}

//+------------------------------------------------------------------+
//|                                 |
//+------------------------------------------------------------------+
void CloseAllPositions() {
  for (int i=OrdersTotal()-1; i>=0; i--) {
    if (OrderSelect(i, SELECT_BY_POS, MODE_TRADES)) {
      if (OrderSymbol()==Symbol()) {
        if (OrderType()==OP_BUY) {
          OrderClose(OrderTicket(), OrderLots(), Bid, Slippage, clCloseBuy);
        }
        if (OrderType()==OP_SELL) {
          OrderClose(OrderTicket(), OrderLots(), Ask, Slippage, clCloseSell);
        }
      }
    }
  }
}

//+------------------------------------------------------------------+
//|                                    |
//+------------------------------------------------------------------+
void Message(string m) {
  Comment(m);
  if (StringLen(m)>0) Print(m);
}
//+------------------------------------------------------------------+

